# File for module specific CMake tests.
#
# .. cmake_function:: add_generated_executable
#
#    .. cmake_param:: UFLFILE
#       :single:
#       :required:
#
#       The UFL file to create the executable from.
#
#    .. cmake_param:: TARGET
#       :single:
#       :required:
#
#       The name given to the added executable target.
#
#    .. cmake_param:: OPERATOR
#       :single:
#
#       The local operator file name to generate. Defaults
#       to a suitably mangled, but not easily readable name.
#
#    .. cmake_param:: DRIVER
#       :single:
#
#       The driver file name to generate. Defaults
#       to a suitably mangled, but not easily readable name.
#
#    .. cmake_param:: MAIN
#
#       The main source file to generate. Defaults
#       to a suitably mangled, but not easily readable name.
#
#    .. cmake_param:: FORM_COMPILER_ARGS
#       :multi:
#       :argname arg:
#
#       Additional arguments as recognized by the form compiler.
#
#    Add an executable to the project that gets automatically
#    generated at configure time with the form compiler uf2pdelab.
#    Regeneration is triggered correctly if the UFL file or the
#    form compiler changed.
#
# .. cmake_variable:: UFL2PDELAB_INTERACTIVE
#
#    If this variable is set, all code generation will be done in
#    interactive mode. This option is interesting in development
#    of the form compiler, but might be quite tedious in production
#    and automated testing.
#

add_custom_target(generation)

# Gather a list of form compiler sources to add as dependencies
# to have correct retriggers of generated executables
if(CMAKE_PROJECT_NAME STREQUAL dune-perftool)
  set(UFL2PDELAB_GLOB_PATTERN "${CMAKE_SOURCE_DIR}/python/*.py")
else()
  # TODO: How do we get to the source here???
endif()
file(GLOB_RECURSE UFL2PDELAB_SOURCES ${UFL2PDELAB_GLOB_PATTERN})

function(add_generated_executable)
  set(OPTIONS)
  set(SINGLE TARGET OPERATOR DRIVER UFLFILE)
  set(MULTI FORM_COMPILER_ARGS)
  include(CMakeParseArguments)
  cmake_parse_arguments(GEN "${OPTIONS}" "${SINGLE}" "${MULTI}" ${ARGN})

  if(GEN_UNPARSED_ARGUMENTS)
    message(FATAL_ERROR "Unrecognized arguments in add_generated_executable. This usually indicates a typo.")
  endif()

  # Apply defaults and enforce requirements
  if(NOT GEN_TARGET)
    message(FATAL_ERROR "Need to specify the TARGET parameter for add_generated_executable")
  endif()
  if(NOT GEN_UFLFILE)
    message(FATAL_ERROR "Need to specify the UFLFILE parameter for add_generated_executable")
  endif()
  if(NOT IS_ABSOLUTE GEN_UFLFILE)
    set(GEN_UFLFILE ${CMAKE_CURRENT_SOURCE_DIR}/${GEN_UFLFILE})
  endif()
  if(NOT GEN_OPERATOR)
    set(GEN_OPERATOR ${GEN_TARGET}_operator.hh)
    set(GEN_OPERATOR ${CMAKE_CURRENT_BINARY_DIR}/${GEN_OPERATOR})
  endif()
  if(NOT GEN_DRIVER)
    set(GEN_DRIVER ${GEN_TARGET}_driver.hh)
    set(GEN_DRIVER ${CMAKE_CURRENT_BINARY_DIR}/${GEN_DRIVER})
  endif()
  if(NOT GEN_MAIN)
    set(GEN_MAIN ${GEN_TARGET}_main.cc)
    set(GEN_MAIN ${CMAKE_CURRENT_BINARY_DIR}/${GEN_MAIN})
  endif()
  if(UFL2PDELAB_INTERACTIVE)
    set(GEN_FORM_COMPILER_ARGS ${GEN_FORM_COMPILER_ARGS} --interactive)
  endif()

  # Write a standard main function
  configure_file(${CMAKE_SOURCE_DIR}/cmake/modules/StandardMain.cmake ${GEN_MAIN})

  add_custom_command(OUTPUT ${GEN_OPERATOR} ${GEN_DRIVER}
                     COMMAND ${CMAKE_BINARY_DIR}/dune-env ufl2pdelab
                             --project-basedir ${CMAKE_BINARY_DIR}
                             --operator-file ${GEN_OPERATOR}
                             --driver-file ${GEN_DRIVER}
                             ${GEN_FORM_COMPILER_ARGS}
                             ${GEN_UFLFILE}
                     DEPENDS ${GEN_UFLFILE} ${UFL2PDELAB_SOURCES}
                     COMMENT "Running ufl2pdelab for the target ${GEN_TARGET}"
                    )

  add_executable(${GEN_TARGET} ${GEN_MAIN} ${GEN_OPERATOR} ${GEN_DRIVER})

  add_dependencies(generation ${GEN_TARGET})
endfunction()

include(GeneratedSystemtests)